import ctypes
from .__library__ import lib, get_error
from .camera import *
from .processing_config import *
from .data import *


class CsfCamera(Camera):
    """
    The csf camera class contains all the logic to generate a stream of depth data
    from a csf file
    """
    def __del__(self):
        if self._ptr:
            lib.tof_csf_camera_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, processing_config = None, file_location = None, ptr = None):
        """
        Construct new csf_camera class
        ----------

        Construct a csf camera from a csf file

        Parameters
        ----------
        processing_config : ProcessingConfig
                A processing config
        file_location : str
                File location of the csf file
        ----------

        Construct a csf camera from a csf file with a processing config generated by
        default_processing from the camera config

        Parameters
        ----------
        file_location : str
                File location of the csf file
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif processing_config != None and file_location != None:
            self._ptr = lib.tof_csf_camera_new(processing_config._ptr, file_location.encode("utf-8"), get_error())
        elif file_location != None:
            self._ptr = lib.tof_csf_camera_new_simple(file_location.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def getFramesAt(self, frame_nr):
        """
        Get frames at the specified position

        Parameters
        ----------
        frame_nr : int
            Which set of frames to process

        Returns
        ----------
        List[Data]
            Data frames
        """
        size = lib.tof_csf_camera_get_frames_at(self._ptr, frame_nr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_csf_camera_get_frames_at(self._ptr, frame_nr, array, size, get_error())
        return [Data(ptr = item) for item in array]

    def getCurrentFrame(self):
        """
        Get the location (count) of the current frame

        Returns
        ----------
        int
            The currenc frame location
        """
        return lib.tof_csf_camera_get_current_frame(self._ptr, get_error())


    def getFrameNumber(self):
        """
        Get the amount of frame lists this csf file can open/generate

        Returns
        ----------
        int
            The number of frames in the csf file
        """
        return lib.tof_csf_camera_get_frame_number(self._ptr, get_error())


    def getFps(self):
        """
        Get the FPS If the FPS is greater than 0 the time between subsequent
        function calls to get_frames or get_frames_at will be limited to the FPS
        This is done by sleeping on the next call to get_frames() if the time
        between calls is smaller than the time duration dictated by the fps.


        Returns
        ----------
        int
            The frames per second
        """
        return lib.tof_csf_camera_get_fps(self._ptr, get_error())


    def setFps(self, fps):
        """
        Set the FPS If the FPS is greater than 0 the time between subsequent
        function calls to get_frames or get_frames_at will be limited to the FPS
        This is done by sleeping on the next call to get_frames() if the time
        between calls is smaller than the time duration dictated by the fps.


        Parameters
        ----------
        fps : int
            The frames per second
        """
        return lib.tof_csf_camera_set_fps(self._ptr, fps, get_error())


